###############################################################################
# script to calculate the efficiency of heatsink/peltier combo
#
# author:	Njaal Brekke
# date:		15.04.2014
# url:		https://ragnablade.myon.no
#
###############################################################################


import numpy as np
from matplotlib import pyplot as plt

###############################################################################
# configuration
###############################################################################

# peltier elements
tec1 = {
	'Name'	: "TEC1-12705",
	'Qmax'	: 43.,
	'dTmax'	: 66.,
	'Imax'	: 5.3,
	'Umax'	: 14.2
	}

tec2 = {
	'Name'	: "TEC1-12703",
	'Qmax'	: 27.72,
	'dTmax'	: 66.,
	'Imax'	: 3.0,
	'Umax'	: 15.4
	}

tec3 = {
	'Name'	: "TEC1-12702",
	'Qmax'	: 18.4,
	'dTmax'	: 66.,
	'Imax'	: 2.0,
	'Umax'	: 15.4
	}
	
# fan/heatsink combo
heatsink1 = {
	'Name'	: "MassCool 5R057B3",
	'C'		: 1./0.674	# heatsink W/C
}

heatsink2 = {
	'Name'	: "CP464-060-060-02-S-100",
	'C'		: 1./1.1	# heatsink W/C
}

heatsink3 = {
	'Name'	: "Startech FAN370PRO",
	'C'		: 1./0.64	# heatsink W/C
}	

# peltier and heatsink to plot
tec = tec1
heatsink = heatsink3

# camera power usage
Iload = 0.2 # canon eos current draw, imaging
Uload = 8.0 # canon eos voltage

# surroundings
T_amb = 0.	# ambient temperature in Celsius

C_amb = 0.15 # W/C transfer from ambient (insulation value)

###############################################################################
# peltier calculation function
###############################################################################

def tec_deltaT(TEC, Chs, Cload, Qload, Itec):
	# read values
	Qmax = TEC['Qmax']
	dTmax = TEC['dTmax']
	Imax = TEC['Imax']
	Umax = TEC['Umax']
	Rtec = Umax/Imax

	P = (Qmax + Imax**2. * Rtec/2.) / Imax # peltier constant
	Ctec = Qmax/dTmax # conductivity of peltier
	#Cload = 0. # conductivity of thermal load to ambient w/c
	
	# tec power
	Qtec = Rtec * Itec**2
	# total heat
	Qtot = Qload + Qtec
	# heatsink to ambient delta temperature
	dThs = Qtot/Chs
	# delta temperature
	#dT = (1-(Qc/Qmax)) * dTmax 

	dTtec = (-P*Itec + Qtec/2. + Qload) / (Ctec + Cload)
	
	return dThs,dTtec

###############################################################################
# main program
###############################################################################
Qload = Iload * Uload	# load power

I_min = 0.
I_max = tec['Imax']

I_tec = np.linspace(I_min,I_max,100,True)
dT_hs,dT_tec = tec_deltaT(tec,heatsink['C'],C_amb,Qload,I_tec)

#fig = plt.figure()
#ax = fig.add_subplot(111)
T_hs = dT_hs+T_amb
T_load = dT_hs+dT_tec+T_amb
T_min_index = np.argmin(T_load)

plt.plot(I_tec,T_hs,label='Heatsink')
plt.plot(I_tec,T_load,label='Sensor')
#plt.plot(I_tec,dT_tec,label='dT tec')
plt.axhline(T_load[T_min_index],color='r',linestyle='--')
plt.axvline(I_tec[T_min_index],color='r',linestyle='--')
plt.legend()
plt.ylim(-50.,60)
plt.ylabel(u'Temperature rel. to ambient (\u00b0C)')
plt.xlabel('Current through TEC (A)')
plt.xlim(0.,5.)
plt.grid(which='both')
name = '%s with %s'%(tec['Name'],heatsink['Name'])
plt.title(name)
plt.savefig('%s.png'%name.replace(' ','_'))
plt.show()